package com.icing.dialer.services

import android.Manifest
import android.content.Context
import android.content.pm.PackageManager
import android.database.Cursor
import android.net.Uri
import android.os.Build
import android.provider.VoicemailContract
import android.telecom.TelecomManager
import android.util.Log
import androidx.core.content.ContextCompat
import io.flutter.plugin.common.MethodCall
import io.flutter.plugin.common.MethodChannel
import java.io.File
import java.io.FileOutputStream

class VoicemailService(private val context: Context) : MethodChannel.MethodCallHandler {
    companion object {
        private const val TAG = "VoicemailService"
    }

    override fun onMethodCall(call: MethodCall, result: MethodChannel.Result) {
        when (call.method) {
            "getVoicemails" -> {
                try {
                    Log.d(TAG, "Getting voicemails...")
                    
                    // Check if app is default dialer
                    if (!isDefaultDialer()) {
                        Log.e(TAG, "App is not the default dialer, cannot access voicemails")
                        result.error("NOT_DEFAULT_DIALER", "App must be set as default dialer to access voicemails", null)
                        return
                    }
                    
                    // Check permissions
                    if (!hasVoicemailPermissions()) {
                        Log.e(TAG, "Missing voicemail permissions")
                        result.error("MISSING_PERMISSIONS", "App doesn't have required voicemail permissions", null)
                        return
                    }
                    
                    val voicemails = getVoicemails()
                    Log.d(TAG, "Found ${voicemails.size} voicemails")
                    result.success(voicemails)
                } catch (e: Exception) {
                    Log.e(TAG, "Error getting voicemails", e)
                    result.error("VOICEMAIL_ERROR", e.message, null)
                }
            }
            "markVoicemailAsRead" -> {
                try {
                    val id = call.argument<String>("id")
                    val isRead = call.argument<Boolean>("isRead") ?: true
                    if (id != null) {
                        val success = markVoicemailAsRead(id, isRead)
                        result.success(success)
                    } else {
                        result.error("INVALID_ARGUMENTS", "Missing voicemail id", null)
                    }
                } catch (e: Exception) {
                    Log.e(TAG, "Error marking voicemail as read", e)
                    result.error("VOICEMAIL_ERROR", e.message, null)
                }
            }
            "deleteVoicemail" -> {
                try {
                    val id = call.argument<String>("id")
                    if (id != null) {
                        val success = deleteVoicemail(id)
                        result.success(success)
                    } else {
                        result.error("INVALID_ARGUMENTS", "Missing voicemail id", null)
                    }
                } catch (e: Exception) {
                    Log.e(TAG, "Error deleting voicemail", e)
                    result.error("VOICEMAIL_ERROR", e.message, null)
                }
            }
            else -> result.notImplemented()
        }
    }

    private fun hasVoicemailPermissions(): Boolean {
        val permissions = arrayOf(
            Manifest.permission.READ_VOICEMAIL,
            Manifest.permission.WRITE_VOICEMAIL,
            Manifest.permission.ADD_VOICEMAIL // Added this permission
        )
        
        return permissions.all {
            ContextCompat.checkSelfPermission(context, it) == PackageManager.PERMISSION_GRANTED
        }
    }

    private fun isDefaultDialer(): Boolean {
        val telecomManager = context.getSystemService(Context.TELECOM_SERVICE) as TelecomManager
        return telecomManager.defaultDialerPackage == context.packageName
    }

    private fun getVoicemails(): List<Map<String, Any?>> {  // Changed return type to Any?
        val voicemails = mutableListOf<Map<String, Any?>>()  // Changed type to Any?
        val uri = VoicemailContract.Voicemails.CONTENT_URI

        val projection = arrayOf(
            VoicemailContract.Voicemails._ID,
            VoicemailContract.Voicemails.NUMBER,
            VoicemailContract.Voicemails.DATE,
            VoicemailContract.Voicemails.DURATION,
            VoicemailContract.Voicemails.HAS_CONTENT,
            VoicemailContract.Voicemails.IS_READ,
            VoicemailContract.Voicemails.SOURCE_PACKAGE
        )

        context.contentResolver.query(
            uri,
            projection,
            null,
            null,
            "${VoicemailContract.Voicemails.DATE} DESC"
        )?.use { cursor ->
            while (cursor.moveToNext()) {
                val id = cursor.getString(cursor.getColumnIndexOrThrow(VoicemailContract.Voicemails._ID))
                val number = cursor.getString(cursor.getColumnIndexOrThrow(VoicemailContract.Voicemails.NUMBER))
                val date = cursor.getLong(cursor.getColumnIndexOrThrow(VoicemailContract.Voicemails.DATE))
                val duration = cursor.getLong(cursor.getColumnIndexOrThrow(VoicemailContract.Voicemails.DURATION))
                val hasContent = cursor.getInt(cursor.getColumnIndexOrThrow(VoicemailContract.Voicemails.HAS_CONTENT)) == 1
                val isRead = cursor.getInt(cursor.getColumnIndexOrThrow(VoicemailContract.Voicemails.IS_READ)) == 1

                var filePath: String? = null
                if (hasContent) {
                    filePath = saveVoicemailToFile(id)
                }

                val voicemail: Map<String, Any?> = mapOf(  // Added explicit type
                    "id" to id,
                    "phoneNumber" to (number ?: "Unknown"),
                    "timestamp" to date,
                    "duration" to (duration / 1000).toInt(), // Convert to seconds
                    "filePath" to filePath,
                    "isRead" to isRead
                )
                voicemails.add(voicemail)
            }
        }

        return voicemails
    }

    private fun saveVoicemailToFile(voicemailId: String): String? {
        val voicemailUri = Uri.withAppendedPath(VoicemailContract.Voicemails.CONTENT_URI, voicemailId)
        
        try {
            val inputStream = context.contentResolver.openInputStream(voicemailUri) ?: return null
            val cacheDir = context.cacheDir
            val voicemailFile = File(cacheDir, "voicemail_$voicemailId.3gp")
            
            FileOutputStream(voicemailFile).use { outputStream ->
                val buffer = ByteArray(4096)
                var bytesRead: Int
                while (inputStream.read(buffer).also { bytesRead = it } != -1) {
                    outputStream.write(buffer, 0, bytesRead)
                }
            }
            
            inputStream.close()
            return voicemailFile.absolutePath
        } catch (e: Exception) {
            Log.e(TAG, "Error saving voicemail to file", e)
            return null
        }
    }

    private fun markVoicemailAsRead(voicemailId: String, isRead: Boolean): Boolean {
        val voicemailUri = Uri.withAppendedPath(VoicemailContract.Voicemails.CONTENT_URI, voicemailId)
        val values = android.content.ContentValues().apply {
            put(VoicemailContract.Voicemails.IS_READ, if (isRead) 1 else 0)
        }
        
        val updated = context.contentResolver.update(voicemailUri, values, null, null)
        return updated > 0
    }

    private fun deleteVoicemail(voicemailId: String): Boolean {
        val voicemailUri = Uri.withAppendedPath(VoicemailContract.Voicemails.CONTENT_URI, voicemailId)
        val deleted = context.contentResolver.delete(voicemailUri, null, null)
        return deleted > 0
    }
}